/*
 *  ======== main.c ========
 */
#include <stdbool.h>
#include <string.h>

#include <xdc/std.h>

#include <xdc/runtime/Error.h>
#include <xdc/runtime/System.h>

#include <ti/sysbios/BIOS.h>

#include <ti/sysbios/knl/Task.h>

/* OSAL Header files */
#include <ti/osal/osal.h>

/* CSL Header files */
#include <ti/csl/csl_chip.h>
#include <ti/csl/cslr_emif16.h>

/* CSL Cache module includes */
#include <ti/csl/csl_cacheAux.h>

#include <ti/board/board.h>

#define hEmif16Cfg     ((CSL_Emif16Regs*)CSL_EMIF16_REGS)

#define ADC_Fifo            0x70000000
#define ADC_CSR0            0x70000010
#define ADC_CfgValue        0x70000020
#define ADC_CSR1            0x70000030

#define DIO_DATA_LW         0x70000200
#define DIO_DATA_HW         0x70000204
#define DIO_CSR             0x70000208

#define DAC0_Data                0x70008000
#define DAC0_CmdAddr             0x70008004
#define DAC0_CSR0                0x70008008
#define DAC0_CSR1                0x7000800C
#define DAC0_PLAYADDR            0x70008010
#define DAC0_TOGGLECNT           0x70008014

#define DAC1_Data                0x7000A000
#define DAC1_CmdAddr             0x7000A004
#define DAC1_CSR0                0x7000A008
#define DAC1_CSR1                0x7000A00C
#define DAC1_PLAYADDR            0x7000A010
#define DAC1_TOGGLECNT           0x7000A014

#define DAC0_Mem                0x7000C000
#define DAC1_Mem                0x70010000

#define DAC0_Data                0x70008000
#define DAC0_CmdAddr             0x70008004
#define DAC0_CSR0                0x70008008
#define DAC0_CSR1                0x7000800C
#define DAC0_PLAYADDR            0x70008010
#define DAC0_TOGGLECNT           0x70008014

#define DAC0_Mem                0x7000C000

uint32_t gADCBuf;

#define ADCStatusReg    (*(volatile UInt16 *)ADC_CSR0)

#define BUF_SZ     64
#define DBUF_SZ     BUF_SZ * 2
#define ADCBUF_SZ   DBUF_SZ * 512

#pragma DATA_ALIGN (dbuf0, 128)
int16_t dbuf0[BUF_SZ];
#pragma DATA_ALIGN (dbuf1, 128)
int16_t dbuf1[BUF_SZ];
#pragma DATA_ALIGN (dbuf2, 128)
int16_t dbuf2[BUF_SZ];
#pragma DATA_ALIGN (dbuf3, 128)
int16_t dbuf3[BUF_SZ];


void dsp_delay(uint32_t loops)
{
    while (loops--) {
        asm("   NOP");
    }
}

void startADC(void)
{
    (*(volatile uint16_t *)ADC_CSR0) = 0x0; //Reset ADC
    dsp_delay(200000);
    (*(volatile uint16_t *)ADC_CSR0) = 0x1; //Reset ADC
    dsp_delay(200000);

    (*(volatile uint16_t *)ADC_CSR1) = 0x190; //set sample rate to 500kHz
    dsp_delay(200000);
    (*(volatile uint16_t *)ADC_CfgValue) = 0x89CD; //set new values
    dsp_delay(200000);
    (*(volatile uint16_t *)ADC_CSR0) = 0x11; //program the adc's
    dsp_delay(200000);
    (*(volatile uint16_t *)ADC_CSR1) = 0xC7; //1Mhz sample rate
    dsp_delay(200000);
    (*(volatile uint16_t *)ADC_CSR0) = 0x3; //start smapling
    dsp_delay(200000);
}

void stopADC(void)
{
    (*(volatile uint16_t *)ADC_CSR0) = 0x1; //stop sampling
}


void acquireADC(void)
{
    uint32_t i=0, d=0;
    uint32_t temp;

    while(1)
    {
        startADC();
        gADCBuf = 0x80000000;

        i = 0;
        d = 0;

        while(d < 512)
        {

            if(!(ADCStatusReg & 0x100)) {

                dbuf0[i] = (*(volatile Int16 *)ADC_Fifo);
                dbuf1[i] = (*(volatile Int16 *)ADC_Fifo);
                dbuf2[i] = (*(volatile Int16 *)ADC_Fifo);
                dbuf3[i++] = (*(volatile Int16 *)ADC_Fifo);

                if(i == BUF_SZ)
                {
                    i=0;
                    d++;

                    temp = gADCBuf;

                    memcpy((void *)gADCBuf, (void *)&dbuf0[0], DBUF_SZ);
                    gADCBuf += ADCBUF_SZ;
                    memcpy((void *)gADCBuf, (void *)&dbuf1[0], DBUF_SZ);
                    gADCBuf += ADCBUF_SZ;
                    memcpy((void *)gADCBuf, (void *)&dbuf2[0], DBUF_SZ);
                    gADCBuf += ADCBUF_SZ;
                    memcpy((void *)gADCBuf, (void *)&dbuf3[0], DBUF_SZ);

                    gADCBuf = temp;

                    gADCBuf += DBUF_SZ;
                    if(gADCBuf >= (0x80000000 + ADCBUF_SZ))
                        gADCBuf = 0x80000000;

                }

            }

        }

        stopADC();
    }

}

void DAC0_Direct (void)
{

    (*(volatile uint16_t *)DAC0_Data) = 0xFFFF;
    (*(volatile uint16_t *)DAC0_CmdAddr) = 0x0030;
    dsp_delay(0x1000);
    (*(volatile uint16_t *)DAC0_Data) = 0x8000;
    (*(volatile uint16_t *)DAC0_CmdAddr) = 0x0030;
    dsp_delay(0x1000);
    (*(volatile uint16_t *)DAC0_Data) = 0x0;
    (*(volatile uint16_t *)DAC0_CmdAddr) = 0x0030;
    dsp_delay(0x1000);
    (*(volatile uint16_t *)DAC0_Data) = 0x3000;
    (*(volatile uint16_t *)DAC0_CmdAddr) = 0x0030;
    dsp_delay(0x1000);
    (*(volatile uint16_t *)DAC0_Data) = 0xA000;
    (*(volatile uint16_t *)DAC0_CmdAddr) = 0x0030;
    dsp_delay(0x1000);
    (*(volatile uint16_t *)DAC0_Data) = 0x100;
    (*(volatile uint16_t *)DAC0_CmdAddr) = 0x0030;
    dsp_delay(0x1000);

}

void DAC0_PlayMem (void)
{
    //write values to DAC memory
    (*(volatile uint16_t *)DAC0_Mem) = 0xFFFF;
    (*(volatile uint16_t *)(DAC0_Mem + 0x4)) = 0x0030;
    (*(volatile uint16_t *)(DAC0_Mem + 0x8)) = 0x0;
    (*(volatile uint16_t *)(DAC0_Mem + 0xC)) = 0x0030;
    (*(volatile uint16_t *)(DAC0_Mem + 0x10)) = 0x4000;
    (*(volatile uint16_t *)(DAC0_Mem + 0x14)) = 0x0030;
    (*(volatile uint16_t *)(DAC0_Mem + 0x18)) = 0x8000;
    (*(volatile uint16_t *)(DAC0_Mem + 0x1C)) = 0x0030;
    (*(volatile uint16_t *)(DAC0_Mem + 0x20)) = 0xA000;
    (*(volatile uint16_t *)(DAC0_Mem + 0x24)) = 0x0030;
    //set last address for play loop
    (*(volatile uint16_t *)DAC0_PLAYADDR) = 0x4;
    //enable DAC memory play
    (*(volatile uint16_t *)DAC0_CSR1) = 0x1;

}


/*
 *  ======== taskFxn ========
 */
Void taskFxn(UArg a0, UArg a1)
{
    int i;

    System_printf("enter taskFxn()\n");

    //Toggle DAC CLR line to initialize DAC
    (*(volatile uint16_t *)DAC0_CSR0) = 0x0;
    dsp_delay(0x1000);
    (*(volatile uint16_t *)DAC0_CSR0) = 0x1;
    dsp_delay(0x4000);

    for(i=0; i<100; i++)
        DAC0_Direct();

    DAC0_PlayMem();

    //set DIO ouput on low 16
    (*(volatile uint16_t *)DIO_CSR) = 0x9;
    (*(volatile uint16_t *)DIO_DATA_LW) = 0xFFFF;

    acquireADC();

    System_printf("exit taskFxn()\n");

    System_flush(); /* force SysMin output to console */
}


bool Board_initSI(void)
{
    Board_initCfg boardCfg;
    Board_STATUS  boardStatus;

    boardCfg = BOARD_INIT_UNLOCK_MMR | BOARD_INIT_PLL | BOARD_INIT_DDR | BOARD_INIT_MODULE_CLOCK;

    boardStatus = Board_init(boardCfg);
    if (boardStatus != BOARD_SOK)
    {
        return (false);
    }

    return (true);
}

/*
 *  ======== main ========
 */
Int main()
{ 
    Task_Handle task;
    Error_Block eb;
    uint32_t i;

    System_printf("enter main()\n");

    if (Board_initSI() == false)
    {
        System_printf("\nBoard_initSI failed!\n");
        return(0);
    }

    CACHE_wbInvAllL2(CACHE_WAIT);

    CACHE_disableCaching(112); //0x70000000 EMIF (FPGA)
    CACHE_setMemRegionInfo(112, 0, 0); //and this one for PFX (prefetchable)

    //DDR - 0x80000000 - 0x82FFFFFF
    for (i=128; i<131; i++)
    {
        CACHE_disableCaching(i);
        CACHE_setMemRegionInfo(i, 0, 0); //and this one for PFX (prefetchable)
    }

    /* EMIF16 setup */
    // This sentence is from the silicon errata
    // Usage Note 25 Performance Degradation for Asynchronous Access Caused by An Unused
    // Feature Enabled in EMIF16 Usage Note
    *(Uint32*)0x20c00008 |= 0x80000000;
    //Set async config register
    //hEmif16Cfg->A0CR = 0x00100081;
    hEmif16Cfg->A0CR = 0x08104081;

    Error_init(&eb);
    task = Task_create(taskFxn, NULL, &eb);
    if (task == NULL) {
        System_printf("Task_create() failed!\n");
        BIOS_exit(0);
    }

    BIOS_start();    /* does not return */
    return(0);
}
